@@ -1,5 +1,6 @@
 use std::fmt;
 use std::io::{BufWriter, Read, Write};
+use std::ops::{Bound, RangeBounds};
 use std::slice::Iter;
 #[cfg(feature = "logging")]
 use std::time::Instant;
@@ -8,6 +9,7 @@ use aho_corasick::AhoCorasick;
 #[cfg(feature = "logging")]
 use log::*;
 use regex_automata::meta::Regex;
+use rustc_hash::FxHashMap;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use crate::compiler::atoms::Atom;
@@ -84,6 +86,19 @@ pub struct Rules {
     /// where the sub-pattern belongs to.
     pub(in crate::compiler) sub_patterns: Vec<(PatternId, SubPattern)>,
 
+    /// Map that associates a `PatternId` to a certain file size bound.
+    ///
+    /// A condition like `filesize < 1000 and $a` only matches if `filesize`
+    /// is less than 1000. Therefore, the pattern `$a` does not need be
+    /// checked for files of size 1000 bytes or larger.
+    ///
+    /// In this case, the map will contain an entry associating `$a` to a
+    /// `FilesizeBounds` value like:
+    ///
+    /// `FilesizeBounds{start: Bound::Unbounded, end: Bound:Excluded(1000)}`.
+    pub(in crate::compiler) filesize_bounds:
+        FxHashMap<PatternId, FilesizeBounds>,
+
     /// Vector that contains the [`SubPatternId`] for sub-patterns that can
     /// match only at a fixed offset within the scanned data. These sub-patterns
     /// are not added to the Aho-Corasick automaton.
@@ -319,9 +334,9 @@ impl Rules {
     ) -> Option<(RuleId, IdentId)> {
         let (target_pattern_id, _) = self.get_sub_pattern(sub_pattern_id);
         for (rule_id, rule) in self.rules.iter().enumerate() {
-            for (ident_id, _, pattern_id, _) in &rule.patterns {
-                if pattern_id == target_pattern_id {
-                    return Some((rule_id.into(), *ident_id));
+            for p in &rule.patterns {
+                if p.pattern_id == *target_pattern_id {
+                    return Some((rule_id.into(), p.ident_id));
                 };
             }
         }
@@ -462,6 +477,14 @@ impl Rules {
     pub(crate) fn wasm_mod(&self) -> &wasmtime::Module {
         self.compiled_wasm_mod.as_ref().unwrap()
     }
+
+    #[inline]
+    pub(crate) fn filesize_bounds(
+        &self,
+        pattern_id: PatternId,
+    ) -> Option<&FilesizeBounds> {
+        self.filesize_bounds.get(&pattern_id)
+    }
 }
 
 #[cfg(feature = "native-code-serialization")]
@@ -547,11 +570,9 @@ impl fmt::Debug for Rules {
             writeln!(f, "  namespace: {namespace}")?;
             writeln!(f, "  name: {name}")?;
             writeln!(f, "  patterns:")?;
-            for (pattern_ident_id, _, pattern_id, _is_private) in
-                &rule.patterns
-            {
-                let ident = self.ident_pool.get(*pattern_ident_id).unwrap();
-                writeln!(f, "    {pattern_id:?} {ident} ")?;
+            for pattern in &rule.patterns {
+                let ident = self.ident_pool.get(pattern.ident_id).unwrap();
+                writeln!(f, "    {:?} {ident} ", pattern.pattern_id)?;
             }
         }
 
@@ -577,31 +598,163 @@ pub(crate) enum MetaValue {
 #[derive(Serialize, Deserialize)]
 pub(crate) struct RuleInfo {
     /// The ID of the namespace the rule belongs to.
-    pub(crate) namespace_id: NamespaceId,
+    pub namespace_id: NamespaceId,
     /// The ID of the rule namespace in the identifiers pool.
-    pub(crate) namespace_ident_id: IdentId,
+    pub namespace_ident_id: IdentId,
     /// The ID of the rule identifier in the identifiers pool.
-    pub(crate) ident_id: IdentId,
+    pub ident_id: IdentId,
     /// Tags associated to the rule.
-    pub(crate) tags: Vec<IdentId>,
+    pub tags: Vec<IdentId>,
     /// Reference to the rule identifier in the source code. This field is
     /// ignored while serializing and deserializing compiles rules, as it
     /// is used only during the compilation phase, but not during the scan
     /// phase.
     #[serde(skip)]
-    pub(crate) ident_ref: CodeLoc,
+    pub ident_ref: CodeLoc,
     /// Metadata associated to the rule.
-    pub(crate) metadata: Vec<(IdentId, MetaValue)>,
+    pub metadata: Vec<(IdentId, MetaValue)>,
     /// Vector with all the patterns defined by this rule. The bool in the
     /// tuple indicates if the pattern is private.
-    pub(crate) patterns: Vec<(IdentId, PatternKind, PatternId, bool)>,
+    pub patterns: Vec<PatternInfo>,
     /// Number of private patterns in the rule. The number of non-private
     /// patterns can be computed as patterns.len - num_private_patterns.
-    pub(crate) num_private_patterns: usize,
+    pub num_private_patterns: usize,
     /// True if the rule is global.
-    pub(crate) is_global: bool,
+    pub is_global: bool,
     /// True if the rule is private.
-    pub(crate) is_private: bool,
+    pub is_private: bool,
+}
+
+/// Information about each of pattern in a rule.
+#[derive(Serialize, Deserialize)]
+pub(crate) struct PatternInfo {
+    /// Unique ID for this pattern.
+    pub pattern_id: PatternId,
+    /// The pattern identifier.
+    pub ident_id: IdentId,
+    /// Indicates if the pattern is text, hex or regexp.
+    pub kind: PatternKind,
+    /// True if the pattern is private.
+    pub is_private: bool,
+}
+
+/// Describes the bounds for `filesize` imposed by a rule condition.
+///
+/// For example, the condition `filesize < 1000 and $a` only matches files
+/// smaller than 10MB. That would be represented by:
+///
+/// ```text
+/// FilesizeBounds { start: Bound::Unbounded, end: Bound::Excluded(1000) }
+/// ```
+///
+/// In contrast, the condition `filesize < 1000 or $a` does not any bounds
+/// to `filesize`, since the use of `or` allows files larger than
+/// 10MB to also match. This case is represented by:
+///
+/// ```text
+/// FilesizeBounds { start: Bound::Unbounded, end: Bound::Unbounded }
+/// ```
+#[derive(Debug, PartialEq, Serialize, Deserialize, Clone, Hash, Eq)]
+pub(crate) struct FilesizeBounds {
+    start: Bound<i64>,
+    end: Bound<i64>,
+}
+
+impl Default for FilesizeBounds {
+    fn default() -> Self {
+        Self { start: Bound::Unbounded, end: Bound::Unbounded }
+    }
+}
+
+impl<T: RangeBounds<i64>> From<T> for FilesizeBounds {
+    fn from(value: T) -> Self {
+        Self {
+            start: value.start_bound().cloned(),
+            end: value.end_bound().cloned(),
+        }
+    }
+}
+
+impl FilesizeBounds {
+    pub fn unbounded(&self) -> bool {
+        matches!(self.start, Bound::Unbounded)
+            && matches!(self.end, Bound::Unbounded)
+    }
+
+    pub fn contains(&self, value: i64) -> bool {
+        let start_ok = match self.start {
+            Bound::Included(start) => value >= start,
+            Bound::Excluded(start) => value > start,
+            Bound::Unbounded => true,
+        };
+
+        let end_ok = match self.end {
+            Bound::Included(end) => value <= end,
+            Bound::Excluded(end) => value < end,
+            Bound::Unbounded => true,
+        };
+
+        start_ok && end_ok
+    }
+    pub fn max_start(&mut self, bound: Bound<i64>) -> &mut Self {
+        match (&self.start, &bound) {
+            (Bound::Included(current), Bound::Included(new)) => {
+                if new > current {
+                    self.start = Bound::Included(*new);
+                }
+            }
+            (Bound::Included(current), Bound::Excluded(new)) => {
+                if new >= current {
+                    self.start = Bound::Excluded(*new);
+                }
+            }
+            (Bound::Excluded(current), Bound::Included(new)) => {
+                if new > current {
+                    self.start = Bound::Included(*new);
+                }
+            }
+            (Bound::Excluded(current), Bound::Excluded(new)) => {
+                if new > current {
+                    self.start = Bound::Excluded(*new);
+                }
+            }
+            (Bound::Unbounded, new) => {
+                self.start = *new;
+            }
+            (_, Bound::Unbounded) => {}
+        }
+        self
+    }
+
+    pub fn min_end(&mut self, bound: Bound<i64>) -> &mut Self {
+        match (&self.end, &bound) {
+            (Bound::Included(current), Bound::Included(new)) => {
+                if new < current {
+                    self.end = Bound::Included(*new);
+                }
+            }
+            (Bound::Included(current), Bound::Excluded(new)) => {
+                if new <= current {
+                    self.end = Bound::Excluded(*new);
+                }
+            }
+            (Bound::Excluded(current), Bound::Included(new)) => {
+                if new < current {
+                    self.end = Bound::Included(*new);
+                }
+            }
+            (Bound::Excluded(current), Bound::Excluded(new)) => {
+                if new < current {
+                    self.end = Bound::Excluded(*new)
+                }
+            }
+            (Bound::Unbounded, new) => {
+                self.end = *new;
+            }
+            (_, Bound::Unbounded) => {}
+        }
+        self
+    }
 }
 
 /// Represents an atom extracted from a pattern and added to the Aho-Corasick
