@@ -34,7 +34,7 @@ where
     I: Iterator<Item = Event>,
 {
     source: &'src [u8],
-    events: Peekable<CSTStream<'src, I>>,
+    events: Peekable<I>,
     errors: Vec<Error>,
     depth: usize,
 }
@@ -43,11 +43,11 @@ impl<'src, I> Builder<'src, I>
 where
     I: Iterator<Item = Event>,
 {
-    pub fn new(cst: CSTStream<'src, I>) -> Self {
+    pub fn new(src: &'src [u8], events: I) -> Self {
         Self {
+            source: src,
+            events: events.peekable(),
             errors: Vec::new(),
-            source: cst.source(),
-            events: cst.peekable(),
             depth: 0,
         }
     }
@@ -64,8 +64,9 @@ where
                     Ok(rule) => items.push(Item::Rule(rule)),
                     // If `rule_decl` returns an error the rule is ignored,
                     // but we try to continue at the next rule declaration,
-                    // import statement, or include statement. The `recover` function discards
-                    // everything until finding the next valid item.
+                    // import statement, or include statement. The `recover`
+                    // function discards everything until finding the next
+                    // valid item.
                     Err(BuilderError::Abort) => self.recover(),
                     Err(BuilderError::MaxDepthReached) => {}
                 },
@@ -94,6 +95,15 @@ where
 
         self.end(SOURCE_FILE).unwrap();
 
+        // After the closing SOURCE_FILE, more error events can follow.
+        for event in self.events {
+            if let Event::Error { message, span } = event {
+                self.errors.push(Error::SyntaxError { message, span });
+            } else {
+                unreachable!("unexpected event: {:?}", event);
+            }
+        }
+
         assert_eq!(self.depth, 0);
 
         AST { items, errors: self.errors }
@@ -164,7 +174,11 @@ where
                 | Event::Begin { kind: INCLUDE_STMT, .. } => break,
                 Event::End { kind: SOURCE_FILE, .. } => break,
                 _ => {
-                    let _ = self.events.next();
+                    if let Some(Event::Error { message, span }) =
+                        self.events.next()
+                    {
+                        self.errors.push(Error::SyntaxError { message, span });
+                    }
                 }
             }
         }
@@ -669,13 +683,15 @@ where
                     modifiers,
                 }))
             }
-            Event::Begin { kind: HEX_PATTERN, .. } => {
+            Event::Begin { kind: HEX_PATTERN, span } => {
+                let span = span.clone();
                 let tokens = self.hex_pattern()?;
                 let modifiers = self.pattern_mods_opt()?;
 
                 Pattern::Hex(Box::new(HexPattern {
-                    identifier,
                     sub_patterns: tokens,
+                    span,
+                    identifier,
                     modifiers,
                 }))
             }
@@ -1145,8 +1161,8 @@ where
             Event::Token { kind: ANY_KW, .. } => {
                 Quantifier::Any { span: self.expect(ANY_KW)? }
             }
-            Event::Begin { kind: PRIMARY_EXPR, .. } => {
-                let expr = self.primary_expr()?;
+            Event::Begin { kind: TERM, .. } => {
+                let expr = self.term()?;
                 self.expect(PERCENT)?;
                 Quantifier::Percentage(expr)
             }
@@ -1243,40 +1259,28 @@ where
         Ok(exprs)
     }
 
-    fn term(&mut self) -> Result<Expr<'src>, BuilderError> {
-        self.begin(TERM)?;
+    fn primary_expr(&mut self) -> Result<Expr<'src>, BuilderError> {
+        self.begin(PRIMARY_EXPR)?;
 
-        let expr = match self.peek() {
+        let mut expr = match self.peek() {
             Event::Begin { kind: FUNC_CALL, .. } => self.func_call(None)?,
-            Event::Begin { kind: PRIMARY_EXPR, .. } => {
-                let mut expr = self.primary_expr()?;
-
-                match self.peek() {
-                    // Array or dictionary lookup.
-                    Event::Token { kind: L_BRACKET, .. } => {
-                        self.expect(L_BRACKET)?;
-                        let index = self.expr()?;
-                        let span = expr.span();
-                        let span = span.combine(&self.expect(R_BRACKET)?);
-                        expr = Expr::Lookup(Box::new(Lookup {
-                            primary: expr,
-                            index,
-                            span,
-                        }))
-                    }
-                    Event::Token { kind: DOT, .. } => {
-                        self.expect(DOT)?;
-                        expr = self.func_call(Some(expr))?;
-                    }
-                    _ => {}
-                }
-
-                expr
+            Event::Token { kind: IDENT, .. } => {
+                Expr::Ident(Box::new(self.identifier()?))
             }
-            _ => unreachable!(),
+            expr => panic!("unexpected {expr:?}"),
         };
 
-        self.end(TERM)?;
+        if let Event::Token { kind: L_BRACKET, .. } = self.peek() {
+            self.expect(L_BRACKET)?;
+            let span = expr.span();
+            let index = self.expr()?;
+            let span = span.combine(&self.expect(R_BRACKET)?);
+
+            expr =
+                Expr::Lookup(Box::new(Lookup { primary: expr, index, span }))
+        }
+
+        self.end(PRIMARY_EXPR)?;
 
         Ok(expr)
     }
@@ -1301,7 +1305,6 @@ where
         let r_paren_span = self.expect(R_PAREN)?;
 
         let expr = Expr::FuncCall(Box::new(FuncCall {
-            span: identifier.span().combine(&r_paren_span),
             args_span: l_paren_span.combine(&r_paren_span),
             object,
             identifier,
@@ -1313,8 +1316,8 @@ where
         Ok(expr)
     }
 
-    fn primary_expr(&mut self) -> Result<Expr<'src>, BuilderError> {
-        self.begin(PRIMARY_EXPR)?;
+    fn term(&mut self) -> Result<Expr<'src>, BuilderError> {
+        self.begin(TERM)?;
 
         let expr = match self.peek() {
             Event::Token { kind: FLOAT_LIT, .. } => {
@@ -1461,25 +1464,89 @@ where
                 self.expect(R_PAREN)?;
                 expr
             }
-            Event::Token { kind: IDENT, .. } => {
-                let mut idents =
-                    vec![Expr::Ident(Box::new(self.identifier()?))];
+            Event::Begin { kind: PRIMARY_EXPR, .. } => {
+                let mut exprs = vec![self.primary_expr()?];
 
                 while let Event::Token { kind: DOT, .. } = self.peek() {
                     self.expect(DOT)?;
-                    idents.push(Expr::Ident(Box::new(self.identifier()?)));
+                    exprs.push(self.primary_expr()?);
                 }
 
-                if idents.len() == 1 {
-                    idents.pop().unwrap()
+                // Consecutive dot-separated expressions can be coalesced
+                // into a single one based on the operands types. For
+                // example `a.b` (identifier . identifier), is merged into
+                // one FieldAccess expression. Similarly, if the left side
+                // is already a FieldAccess, and the right is an identifier,
+                // the identifier is simply added to the operand list of
+                // the FieldAccess. There are more cases, all covered below.
+                let mut coalesced: Vec<Expr> = exprs
+                    .into_iter()
+                    .coalesce(|x, y| match (x, y) {
+                        // Two consecutive identifiers, this is a field access.
+                        (left @ Expr::Ident(_), right @ Expr::Ident(_)) => {
+                            Ok(Expr::FieldAccess(Box::new(NAryExpr::from(
+                                vec![left, right],
+                            ))))
+                        }
+                        // A field access followed by some identifier, add the
+                        // identifier as another operator of the field access.
+                        (
+                            Expr::FieldAccess(mut field_access),
+                            ident @ Expr::Ident(_),
+                        ) => {
+                            field_access.operands.push(ident);
+                            Ok(Expr::FieldAccess(field_access))
+                        }
+                        //  A field access followed by a function call, the
+                        // field access is the target object for the function.
+                        (
+                            field_access @ Expr::FieldAccess(_),
+                            Expr::FuncCall(mut func_call),
+                        ) => {
+                            func_call.object = Some(field_access);
+                            Ok(Expr::FuncCall(func_call))
+                        }
+                        // An identifier followed by a function call, the
+                        // identifier is the target object for the function.
+                        (
+                            ident @ Expr::Ident(_),
+                            Expr::FuncCall(mut func_call),
+                        ) => {
+                            func_call.object = Some(ident);
+                            Ok(Expr::FuncCall(func_call))
+                        }
+                        // An identifier followed by a lookup expression.
+                        (ident @ Expr::Ident(_), Expr::Lookup(mut lookup)) => {
+                            lookup.span = ident.span().combine(&lookup.span);
+                            lookup.primary = Expr::FieldAccess(Box::new(
+                                NAryExpr::from(vec![ident, lookup.primary]),
+                            ));
+                            Ok(Expr::Lookup(lookup))
+                        }
+                        // Field access followed by a lookup expression.
+                        (
+                            Expr::FieldAccess(mut fa),
+                            Expr::Lookup(mut lookup),
+                        ) => {
+                            fa.operands.push(lookup.primary);
+                            lookup.span = fa.span().combine(&lookup.span);
+                            lookup.primary = Expr::FieldAccess(fa);
+                            Ok(Expr::Lookup(lookup))
+                        }
+                        (x, y) => Err((x, y)),
+                    })
+                    .collect();
+
+                if coalesced.len() == 1 {
+                    coalesced.pop().unwrap()
                 } else {
-                    Expr::FieldAccess(Box::new(NAryExpr::from(idents)))
+                    Expr::FieldAccess(Box::new(NAryExpr::from(coalesced)))
                 }
             }
             event => panic!("unexpected {event:?}"),
         };
 
-        self.end(PRIMARY_EXPR)?;
+        self.end(TERM)?;
 
         Ok(expr)
     }
@@ -1712,14 +1779,12 @@ where
         let without_quotes = &literal[num_quotes..literal.len() - num_quotes];
 
         // Check if the string contains some backslash.
-        let backslash_pos = if let Some(backslash_pos) =
-            without_quotes.find('\\')
-        {
+        let first_backslash = if let Some(pos) = without_quotes.find('\\') {
             if !allow_escape_char {
                 self.errors.push(Error::UnexpectedEscapeSequence(span));
                 return Err(BuilderError::Abort);
             }
-            backslash_pos
+            pos
         } else {
             // If the literal does not contain a backslash it can't contain escaped
             // characters, the literal is exactly as it appears in the source code.
@@ -1730,13 +1795,13 @@ where
 
         // Initially the result is a copy of the literal string up to the first
         // backslash found.
-        let mut result = BString::from(&without_quotes[..backslash_pos]);
+        let mut result = BString::from(&without_quotes[..first_backslash]);
 
         // Process the remaining part of the literal, starting at the backslash.
-        let without_quotes = &without_quotes[backslash_pos..];
-        let mut chars = without_quotes.char_indices();
+        let remaining = &without_quotes[first_backslash..];
+        let mut chars = remaining.char_indices();
 
-        while let Some((backslash_pos, b)) = chars.next() {
+        while let Some((backslash, b)) = chars.next() {
             match b {
                 // The backslash indicates an escape sequence.
                 '\\' => {
@@ -1757,42 +1822,40 @@ where
                         't' => result.push(b'\t'),
                         '0' => result.push(b'\0'),
                         '"' => result.push(b'"'),
-                        'x' => match (chars.next(), chars.next()) {
-                            (Some((start, _)), Some((end, _))) => {
-                                if let Ok(hex_value) = u8::from_str_radix(
-                                    &without_quotes[start..=end],
-                                    16,
-                                ) {
+                        'x' => {
+                            match (chars.next(), chars.next()) {
+                                (
+                                    Some((start, first_char)),
+                                    Some((end, second_char)),
+                                ) if first_char.is_ascii_hexdigit()
+                                    && second_char.is_ascii_hexdigit() =>
+                                {
+                                    let hex_value = u8::from_str_radix(
+                                        &remaining[start..=end],
+                                        16,
+                                    )
+                                    .unwrap();
+
                                     result.push(hex_value);
-                                } else {
-                                    self.errors.push(
-                                        Error::InvalidEscapeSequence {
-                                            message: format!(
-                                                r"invalid hex value `{}` after `\x`",
-                                                &without_quotes[start..=end]
-                                            ),
-                                            span: string_span
-                                                .subspan(start, end + 1),
-                                        }
-                                    );
-                                    return Err(BuilderError::Abort);
                                 }
-                            }
-                            _ => {
-                                self.errors
+                                _ => {
+                                    let (escaped_char_pos, _) = escaped_char;
+
+                                    self.errors
                                     .push(Error::InvalidEscapeSequence {
                                     message:
                                         r"expecting two hex digits after `\x`"
                                             .to_string(),
-                                    span: string_span.subspan(
-                                        backslash_pos,
-                                        escaped_char.0 + 1,
+                                    span: string_span.offset(first_backslash as isize).subspan(
+                                        backslash,
+                                        escaped_char_pos + 1,
                                     ),
                                 });
 
-                                return Err(BuilderError::Abort);
+                                    return Err(BuilderError::Abort);
+                                }
                             }
-                        },
+                        }
                         _ => {
                             let (escaped_char_pos, escaped_char) =
                                 escaped_char;
@@ -1803,13 +1866,12 @@ where
                             self.errors.push(Error::InvalidEscapeSequence {
                                 message: format!(
                                     "invalid escape sequence `{}`",
-                                    &without_quotes
-                                        [backslash_pos..escaped_char_end_pos]
-                                ),
-                                span: string_span.subspan(
-                                    backslash_pos,
-                                    escaped_char_end_pos,
+                                    &remaining
+                                        [backslash..escaped_char_end_pos]
                                 ),
+                                span: string_span
+                                    .offset(first_backslash as isize)
+                                    .subspan(backslash, escaped_char_end_pos),
                             });
 
                             return Err(BuilderError::Abort);
