@@ -453,39 +453,39 @@ impl<'src> ParserImpl<'src> {
                 self.end();
                 self.recover();
                 return self;
-            } else {
-                let token_span = token.span();
-                let token_id = token.id();
+            }
 
+            let token_span = token.span();
+            let token_id = token.id();
+
+            // If there were previous errors, flush those errors and
+            // don't produce new ones, but if no previous error exist
+            // then create an error that tells that we are expecting
+            // any of the tokens in the recovery set.
+            if self.pending_errors.is_empty() {
+                self.end();
+                self.trivia();
+                self.begin(ERROR);
+                self.bump();
+
+                let (actual_token_id, expected) =
+                    self.expected_token_errors.entry(token_span).or_default();
+
+                *actual_token_id = token_id;
+
+                expected.extend(
+                    recovery_set.token_ids().map(|token| token.description()),
+                );
+
+                self.handle_errors();
+            } else {
                 self.trivia();
                 self.bump();
-                self.set_state(State::Failure);
-
-                // If there were previous errors, flush those errors and
-                // don't produce new ones, but if no previous error exist
-                // then create an error that tells that we are expecting
-                // any of the tokens in the recovery set.
-                if self.pending_errors.is_empty() {
-                    let (actual_token_id, expected) = self
-                        .expected_token_errors
-                        .entry(token_span)
-                        .or_default();
-
-                    *actual_token_id = token_id;
-
-                    expected.extend(
-                        recovery_set
-                            .token_ids()
-                            .map(|token| token.description()),
-                    );
-
-                    self.handle_errors();
-                } else {
-                    self.flush_errors();
-                }
+                self.flush_errors();
             }
         }
 
+        // Consume any token that is not in the recovery set.
         while let Some(token) = self.peek_non_trivia() {
             if recovery_set.contains(token).is_some() {
                 break;
@@ -1457,7 +1457,7 @@ impl ParserImpl<'_> {
     ///
     /// ```text
     /// EXPR := (
-    ///    TERM  ( (arithmetic_op | bitwise_op | `.`) TERM )*
+    ///    TERM  ( (arithmetic_op | bitwise_op) TERM )*
     /// )
     /// ``
     fn expr(&mut self) -> &mut Self {
@@ -1475,8 +1475,7 @@ impl ParserImpl<'_> {
                             | SHR
                             | BITWISE_AND
                             | BITWISE_OR
-                            | BITWISE_XOR
-                            | DOT),
+                            | BITWISE_XOR),
                         Some("operator"),
                     )
                     .then(|p| p.term())
@@ -1485,37 +1484,6 @@ impl ParserImpl<'_> {
         })
     }
 
-    /// Parses a term.
-    ///
-    /// ```text
-    /// TERM := (
-    ///     FUNC_CALL |
-    ///     PRIMARY_EXPR
-    ///     (
-    ///        `[` EXPR `]` | `.` FUNC_CALL
-    ///     )?
-    /// )
-    /// ``
-    fn term(&mut self) -> &mut Self {
-        self.begin(TERM)
-            .begin_alt()
-            .alt(|p| p.func_call())
-            .alt(|p| {
-                p.primary_expr().opt(|p| {
-                    p.begin_alt()
-                        .alt(|p| {
-                            p.expect(t!(L_BRACKET))
-                                .expr()
-                                .expect(t!(R_BRACKET))
-                        })
-                        .alt(|p| p.expect(t!(DOT)).then(|p| p.func_call()))
-                        .end_alt()
-                })
-            })
-            .end_alt()
-            .end()
-    }
-
     /// Parses a function call.
     ///
     /// ```text
@@ -1523,7 +1491,7 @@ impl ParserImpl<'_> {
     /// ``
     fn func_call(&mut self) -> &mut Self {
         self.begin(FUNC_CALL)
-            .expect_d(t!(IDENT), Some("expression"))
+            .expect(t!(IDENT))
             .expect(t!(L_PAREN))
             .opt(|p| {
                 p.boolean_expr().zero_or_more(|p| {
@@ -1550,10 +1518,10 @@ impl ParserImpl<'_> {
             .end()
     }
 
-    /// Parsers a primary expression.
+    /// Parses a term.
     ///
     /// ```text
-    /// PRIMARY_EXPR := (
+    /// TERM := (
     ///     FLOAT_LIT                          |
     ///     INTEGER_LIT                        |
     ///     STRING_LIT                         |
@@ -1566,14 +1534,14 @@ impl ParserImpl<'_> {
     ///     `-` TERM                           |
     ///     `~` TERM                           |
     ///     `(` EXPR `)`                       |
-    ///     IDENT (`.` IDENT !`(` )*
+    ///     PRIMARY_EXPR ( `.` PRIMARY_EXPR )*
     /// )
     /// ``
-    fn primary_expr(&mut self) -> &mut Self {
+    fn term(&mut self) -> &mut Self {
         const DESC: Option<&'static str> = Some("expression");
 
-        self.cached(PRIMARY_EXPR, |p| {
-            p.begin(PRIMARY_EXPR)
+        self.cached(TERM, |p| {
+            p.begin(TERM)
                 .begin_alt()
                 .alt(|p| {
                     p.expect_d(
@@ -1604,10 +1572,8 @@ impl ParserImpl<'_> {
                         .expect(t!(R_PAREN))
                 })
                 .alt(|p| {
-                    p.expect_d(t!(IDENT), DESC).zero_or_more(|p| {
-                        p.expect(t!(DOT))
-                            .expect(t!(IDENT))
-                            .not(|p| p.expect(t!(L_PAREN)))
+                    p.primary_expr().zero_or_more(|p| {
+                        p.expect_d(t!(DOT), DESC).primary_expr()
                     })
                 })
                 .end_alt()
@@ -1615,6 +1581,29 @@ impl ParserImpl<'_> {
         })
     }
 
+    /// Parses a primary expression.
+    ///
+    /// ```text
+    /// PRIMARY_EXPR : = (
+    ///    FUNC_CALL  |
+    ///    IDENT `[` EXPR `]`
+    ///    IDENT |
+    /// ```
+    fn primary_expr(&mut self) -> &mut Self {
+        self.begin(PRIMARY_EXPR)
+            .begin_alt()
+            .alt(|p| p.func_call())
+            .alt(|p| {
+                p.expect(t!(IDENT))
+                    .expect(t!(L_BRACKET))
+                    .expr()
+                    .expect(t![R_BRACKET])
+            })
+            .alt(|p| p.expect(t!(IDENT)))
+            .end_alt()
+            .end()
+    }
+
     /// Parses `for` expression.
     ///
     /// ```text
@@ -1626,7 +1615,7 @@ impl ParserImpl<'_> {
     /// ``
     fn for_expr(&mut self) -> &mut Self {
         self.begin(FOR_EXPR)
-            .expect(t!(FOR_KW))
+            .expect_d(t!(FOR_KW), Some("expression"))
             .then(|p| p.quantifier())
             .begin_alt()
             .alt(|p| {
@@ -1689,7 +1678,7 @@ impl ParserImpl<'_> {
     /// ```
     fn with_expr(&mut self) -> &mut Self {
         self.begin(WITH_EXPR)
-            .expect(t!(WITH_KW))
+            .expect_d(t!(WITH_KW), Some("expression"))
             .then(|p| p.with_declarations())
             .expect(t!(COLON))
             .expect(t!(L_PAREN))
@@ -1732,19 +1721,21 @@ impl ParserImpl<'_> {
     ///     `all`                           |
     ///     `none`                          |
     ///     `any`                           |
-    ///     (INTEGER_LIT | FLOAT_LIT ) `%`  |
+    ///     TERM `%`                        |
     ///     EXPR !`%`
     /// )
     /// ```
     fn quantifier(&mut self) -> &mut Self {
         self.begin(QUANTIFIER)
             .begin_alt()
-            .alt(|p| p.expect(t!(ALL_KW | NONE_KW | ANY_KW)))
-            // Quantifier can be either a primary expression followed by a %,
-            // or an expression not followed by %. We can't make it an expression
-            // followed by an optional % because that leads to ambiguity, as
-            // expressions can contain the % operator (mod).
-            .alt(|p| p.primary_expr().expect(t!(PERCENT)))
+            .alt(|p| {
+                p.expect_d(t!(ALL_KW | NONE_KW | ANY_KW), Some("expression"))
+            })
+            // Quantifier can be either a term followed by a %, or an expression
+            // not followed by %. We can't make it an expression followed by an
+            // optional % because that leads to ambiguity, as expressions can
+            // contain the % operator (mod).
+            .alt(|p| p.term().expect(t!(PERCENT)))
             .alt(|p| p.expr().not(|p| p.expect(t!(PERCENT))))
             .end_alt()
             .end()
