@@ -24,7 +24,7 @@ fn keywords() {
 #[test]
 fn identifiers() {
     let mut lexer = super::Tokenizer::new(
-        "foo _bar baz0 qux_1 $ $_ $foo @foo #foo !foo".as_bytes(),
+        "foo _bar baz0 qux_1 $ $_ $foo @foo #foo !foo _123".as_bytes(),
     );
 
     assert_eq!(lexer.next_token(), Some(Token::IDENT(Span(0..3))));
@@ -46,6 +46,8 @@ fn identifiers() {
     assert_eq!(lexer.next_token(), Some(Token::PATTERN_COUNT(Span(35..39))));
     assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(39..40))));
     assert_eq!(lexer.next_token(), Some(Token::PATTERN_LENGTH(Span(40..44))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(44..45))));
+    assert_eq!(lexer.next_token(), Some(Token::IDENT(Span(45..49))));
 
     assert_eq!(lexer.next_token(), None);
 }
@@ -75,16 +77,37 @@ fn integer_literals() {
     assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(9..10))));
     assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(10..14))));
     assert_eq!(lexer.next_token(), None);
+
+    let mut lexer = super::Tokenizer::new(
+        r#"1_0 0x1_0 0o1_0 1__0 0x1__0 0o1__0"#.as_bytes(),
+    );
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(0..3))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(3..4))));
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(4..9))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(9..10))));
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(10..15))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(15..16))));
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(16..20))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(20..21))));
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(21..27))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(27..28))));
+    assert_eq!(lexer.next_token(), Some(Token::INTEGER_LIT(Span(28..34))));
+    assert_eq!(lexer.next_token(), None);
 }
 
 #[test]
 fn float_literals() {
-    let mut lexer = super::Tokenizer::new(r#"3.14 10.0 1.0"#.as_bytes());
+    let mut lexer =
+        super::Tokenizer::new(r#"3.14 10.0 1.0 1_0.0_1 1__0.0__1"#.as_bytes());
     assert_eq!(lexer.next_token(), Some(Token::FLOAT_LIT(Span(0..4))));
     assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(4..5))));
     assert_eq!(lexer.next_token(), Some(Token::FLOAT_LIT(Span(5..9))));
     assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(9..10))));
     assert_eq!(lexer.next_token(), Some(Token::FLOAT_LIT(Span(10..13))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(13..14))));
+    assert_eq!(lexer.next_token(), Some(Token::FLOAT_LIT(Span(14..21))));
+    assert_eq!(lexer.next_token(), Some(Token::WHITESPACE(Span(21..22))));
+    assert_eq!(lexer.next_token(), Some(Token::FLOAT_LIT(Span(22..31))));
     assert_eq!(lexer.next_token(), None);
 }
 
