@@ -5,10 +5,9 @@ fn main() {
     Codegen::new()
         .pure()
         .cargo_out_dir("protos")
-        .include("../proto/src")
         .include("src/tests")
         .include("src")
         .input("src/tests/test.proto")
-        .input("../proto/src/yara.proto")
+        .input("src/yara.proto")
         .run_from_script();
 }
